@@ -2,11 +2,15 @@
   FUSE: Filesystem in Userspace
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
+  Implementation of the single-threaded FUSE session loop.
+
   This program can be distributed under the terms of the GNU LGPLv2.
   See the file COPYING.LIB
 */
 
+#include "config.h"
 #include "fuse_lowlevel.h"
+#include "fuse_i.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -15,32 +19,24 @@
 int fuse_session_loop(struct fuse_session *se)
 {
 	int res = 0;
-	struct fuse_chan *ch = fuse_session_next_chan(se, NULL);
-	size_t bufsize = fuse_chan_bufsize(ch);
-	char *buf = (char *) malloc(bufsize);
-	if (!buf) {
-		fprintf(stderr, "fuse: failed to allocate read buffer\n");
-		return -1;
-	}
+	struct fuse_buf fbuf = {
+		.mem = NULL,
+	};
 
 	while (!fuse_session_exited(se)) {
-		struct fuse_chan *tmpch = ch;
-		struct fuse_buf fbuf = {
-			.mem = buf,
-			.size = bufsize,
-		};
-
-		res = fuse_session_receive_buf(se, &fbuf, &tmpch);
+		res = fuse_session_receive_buf_int(se, &fbuf, NULL);
 
 		if (res == -EINTR)
 			continue;
 		if (res <= 0)
 			break;
 
-		fuse_session_process_buf(se, &fbuf, tmpch);
+		fuse_session_process_buf_int(se, &fbuf, NULL);
 	}
 
-	free(buf);
+	free(fbuf.mem);
+	if(se->error != 0)
+		res = se->error;
 	fuse_session_reset(se);
-	return res < 0 ? -1 : 0;
+	return res;
 }
