@@ -6,7 +6,9 @@
   See the file COPYING.LIB
 */
 
-#define FUSE_USE_VERSION 26
+#define FUSE_USE_VERSION 30
+
+#include <config.h>
 
 #include <fuse.h>
 #include <stdio.h>
@@ -50,26 +52,14 @@ static int subdir_addpath(struct subdir *d, const char *path, char **newpathp)
 	return 0;
 }
 
-static int subdir_getattr(const char *path, struct stat *stbuf)
+static int subdir_getattr(const char *path, struct stat *stbuf,
+			  struct fuse_file_info *fi)
 {
 	struct subdir *d = subdir_get();
 	char *newpath;
 	int err = subdir_addpath(d, path, &newpath);
 	if (!err) {
-		err = fuse_fs_getattr(d->next, newpath, stbuf);
-		free(newpath);
-	}
-	return err;
-}
-
-static int subdir_fgetattr(const char *path, struct stat *stbuf,
-			   struct fuse_file_info *fi)
-{
-	struct subdir *d = subdir_get();
-	char *newpath;
-	int err = subdir_addpath(d, path, &newpath);
-	if (!err) {
-		err = fuse_fs_fgetattr(d->next, newpath, stbuf, fi);
+		err = fuse_fs_getattr(d->next, newpath, stbuf, fi);
 		free(newpath);
 	}
 	return err;
@@ -179,14 +169,15 @@ static int subdir_opendir(const char *path, struct fuse_file_info *fi)
 
 static int subdir_readdir(const char *path, void *buf,
 			  fuse_fill_dir_t filler, off_t offset,
-			  struct fuse_file_info *fi)
+			  struct fuse_file_info *fi,
+			  enum fuse_readdir_flags flags)
 {
 	struct subdir *d = subdir_get();
 	char *newpath;
 	int err = subdir_addpath(d, path, &newpath);
 	if (!err) {
 		err = fuse_fs_readdir(d->next, newpath, buf, filler, offset,
-				      fi);
+				      fi, flags);
 		free(newpath);
 	}
 	return err;
@@ -264,7 +255,7 @@ static int subdir_symlink(const char *from, const char *path)
 	return err;
 }
 
-static int subdir_rename(const char *from, const char *to)
+static int subdir_rename(const char *from, const char *to, unsigned int flags)
 {
 	struct subdir *d = subdir_get();
 	char *newfrom;
@@ -273,7 +264,7 @@ static int subdir_rename(const char *from, const char *to)
 	if (!err) {
 		err = subdir_addpath(d, to, &newto);
 		if (!err) {
-			err = fuse_fs_rename(d->next, newfrom, newto);
+			err = fuse_fs_rename(d->next, newfrom, newto, flags);
 			free(newto);
 		}
 		free(newfrom);
@@ -298,62 +289,53 @@ static int subdir_link(const char *from, const char *to)
 	return err;
 }
 
-static int subdir_chmod(const char *path, mode_t mode)
+static int subdir_chmod(const char *path, mode_t mode,
+			struct fuse_file_info *fi)
 {
 	struct subdir *d = subdir_get();
 	char *newpath;
 	int err = subdir_addpath(d, path, &newpath);
 	if (!err) {
-		err = fuse_fs_chmod(d->next, newpath, mode);
+		err = fuse_fs_chmod(d->next, newpath, mode, fi);
 		free(newpath);
 	}
 	return err;
 }
 
-static int subdir_chown(const char *path, uid_t uid, gid_t gid)
+static int subdir_chown(const char *path, uid_t uid, gid_t gid,
+			struct fuse_file_info *fi)
 {
 	struct subdir *d = subdir_get();
 	char *newpath;
 	int err = subdir_addpath(d, path, &newpath);
 	if (!err) {
-		err = fuse_fs_chown(d->next, newpath, uid, gid);
+		err = fuse_fs_chown(d->next, newpath, uid, gid, fi);
 		free(newpath);
 	}
 	return err;
 }
 
-static int subdir_truncate(const char *path, off_t size)
+static int subdir_truncate(const char *path, off_t size,
+			   struct fuse_file_info *fi)
 {
 	struct subdir *d = subdir_get();
 	char *newpath;
 	int err = subdir_addpath(d, path, &newpath);
 	if (!err) {
-		err = fuse_fs_truncate(d->next, newpath, size);
+		err = fuse_fs_truncate(d->next, newpath, size, fi);
 		free(newpath);
 	}
 	return err;
 }
 
-static int subdir_ftruncate(const char *path, off_t size,
-			    struct fuse_file_info *fi)
+static int subdir_utimens(const char *path, const struct timespec ts[2],
+			  struct fuse_file_info *fi)
 {
 	struct subdir *d = subdir_get();
 	char *newpath;
 	int err = subdir_addpath(d, path, &newpath);
 	if (!err) {
-		err = fuse_fs_ftruncate(d->next, newpath, size, fi);
-		free(newpath);
-	}
-	return err;
-}
-
-static int subdir_utimens(const char *path, const struct timespec ts[2])
-{
-	struct subdir *d = subdir_get();
-	char *newpath;
-	int err = subdir_addpath(d, path, &newpath);
-	if (!err) {
-		err = fuse_fs_utimens(d->next, newpath, ts);
+		err = fuse_fs_utimens(d->next, newpath, ts, fi);
 		free(newpath);
 	}
 	return err;
@@ -560,10 +542,13 @@ static int subdir_bmap(const char *path, size_t blocksize, uint64_t *idx)
 	return err;
 }
 
-static void *subdir_init(struct fuse_conn_info *conn)
+static void *subdir_init(struct fuse_conn_info *conn,
+			 struct fuse_config *cfg)
 {
 	struct subdir *d = subdir_get();
-	fuse_fs_init(d->next, conn);
+	fuse_fs_init(d->next, conn, cfg);
+	/* Don't touch cfg->nullpath_ok, we can work with
+	   either */
 	return d;
 }
 
@@ -579,7 +564,6 @@ static const struct fuse_operations subdir_oper = {
 	.destroy	= subdir_destroy,
 	.init		= subdir_init,
 	.getattr	= subdir_getattr,
-	.fgetattr	= subdir_fgetattr,
 	.access		= subdir_access,
 	.readlink	= subdir_readlink,
 	.opendir	= subdir_opendir,
@@ -595,7 +579,6 @@ static const struct fuse_operations subdir_oper = {
 	.chmod		= subdir_chmod,
 	.chown		= subdir_chown,
 	.truncate	= subdir_truncate,
-	.ftruncate	= subdir_ftruncate,
 	.utimens	= subdir_utimens,
 	.create		= subdir_create,
 	.open		= subdir_open,
@@ -613,9 +596,6 @@ static const struct fuse_operations subdir_oper = {
 	.lock		= subdir_lock,
 	.flock		= subdir_flock,
 	.bmap		= subdir_bmap,
-
-	.flag_nullpath_ok = 1,
-	.flag_nopath = 1,
 };
 
 static const struct fuse_opt subdir_opts[] = {
@@ -629,7 +609,7 @@ static const struct fuse_opt subdir_opts[] = {
 
 static void subdir_help(void)
 {
-	fprintf(stderr,
+	printf(
 "    -o subdir=DIR	    prepend this directory to all paths (mandatory)\n"
 "    -o [no]rellinks	    transform absolute symlinks to relative\n");
 }
