@@ -2,6 +2,8 @@
   FUSE: Filesystem in Userspace
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
+  Architecture specific file system mounting (Linux).
+
   This program can be distributed under the terms of the GNU LGPLv2.
   See the file COPYING.LIB.
 */
@@ -10,7 +12,6 @@
 #include "fuse_i.h"
 #include "fuse_misc.h"
 #include "fuse_opt.h"
-#include "fuse_common_compat.h"
 #include "mount_util.h"
 
 #include <stdio.h>
@@ -29,17 +30,17 @@
 #ifdef __NetBSD__
 #include <perfuse.h>
 
-#define MS_RDONLY 	MNT_RDONLY
-#define MS_NOSUID 	MNT_NOSUID
-#define MS_NODEV 	MNT_NODEV
-#define MS_NOEXEC 	MNT_NOEXEC
-#define MS_SYNCHRONOUS 	MNT_SYNCHRONOUS
-#define MS_NOATIME 	MNT_NOATIME
+#define MS_RDONLY	MNT_RDONLY
+#define MS_NOSUID	MNT_NOSUID
+#define MS_NODEV	MNT_NODEV
+#define MS_NOEXEC	MNT_NOEXEC
+#define MS_SYNCHRONOUS	MNT_SYNCHRONOUS
+#define MS_NOATIME	MNT_NOATIME
 
 #define umount2(mnt, flags) unmount(mnt, (flags == 2) ? MNT_FORCE : 0)
 #endif
 
-#define FUSERMOUNT_PROG		"fusermount"
+#define FUSERMOUNT_PROG		"fusermount3"
 #define FUSE_COMMFD_ENV		"_FUSE_COMMFD"
 
 #ifndef HAVE_FORK
@@ -56,18 +57,13 @@ enum {
 	KEY_FUSERMOUNT_OPT,
 	KEY_SUBTYPE_OPT,
 	KEY_MTAB_OPT,
-	KEY_ALLOW_ROOT,
+	KEY_ALLOW_OTHER,
 	KEY_RO,
-	KEY_HELP,
-	KEY_VERSION,
 };
 
 struct mount_opts {
 	int allow_other;
-	int allow_root;
-	int ishelp;
 	int flags;
-	int nonempty;
 	int auto_unmount;
 	int blkdev;
 	char *fsname;
@@ -76,30 +72,30 @@ struct mount_opts {
 	char *mtab_opts;
 	char *fusermount_opts;
 	char *kernel_opts;
+	unsigned max_read;
 };
 
 #define FUSE_MOUNT_OPT(t, p) { t, offsetof(struct mount_opts, p), 1 }
 
 static const struct fuse_opt fuse_mount_opts[] = {
 	FUSE_MOUNT_OPT("allow_other",		allow_other),
-	FUSE_MOUNT_OPT("allow_root",		allow_root),
-	FUSE_MOUNT_OPT("nonempty",		nonempty),
 	FUSE_MOUNT_OPT("blkdev",		blkdev),
 	FUSE_MOUNT_OPT("auto_unmount",		auto_unmount),
 	FUSE_MOUNT_OPT("fsname=%s",		fsname),
+	FUSE_MOUNT_OPT("max_read=%u",		max_read),
 	FUSE_MOUNT_OPT("subtype=%s",		subtype),
 	FUSE_OPT_KEY("allow_other",		KEY_KERN_OPT),
-	FUSE_OPT_KEY("allow_root",		KEY_ALLOW_ROOT),
-	FUSE_OPT_KEY("nonempty",		KEY_FUSERMOUNT_OPT),
 	FUSE_OPT_KEY("auto_unmount",		KEY_FUSERMOUNT_OPT),
 	FUSE_OPT_KEY("blkdev",			KEY_FUSERMOUNT_OPT),
 	FUSE_OPT_KEY("fsname=",			KEY_FUSERMOUNT_OPT),
 	FUSE_OPT_KEY("subtype=",		KEY_SUBTYPE_OPT),
-	FUSE_OPT_KEY("large_read",		KEY_KERN_OPT),
 	FUSE_OPT_KEY("blksize=",		KEY_KERN_OPT),
 	FUSE_OPT_KEY("default_permissions",	KEY_KERN_OPT),
+	FUSE_OPT_KEY("context=",		KEY_KERN_OPT),
+	FUSE_OPT_KEY("fscontext=",		KEY_KERN_OPT),
+	FUSE_OPT_KEY("defcontext=",		KEY_KERN_OPT),
+	FUSE_OPT_KEY("rootcontext=",		KEY_KERN_OPT),
 	FUSE_OPT_KEY("max_read=",		KEY_KERN_OPT),
-	FUSE_OPT_KEY("max_read=",		FUSE_OPT_KEY_KEEP),
 	FUSE_OPT_KEY("user=",			KEY_MTAB_OPT),
 	FUSE_OPT_KEY("-r",			KEY_RO),
 	FUSE_OPT_KEY("ro",			KEY_KERN_FLAG),
@@ -115,35 +111,16 @@ static const struct fuse_opt fuse_mount_opts[] = {
 	FUSE_OPT_KEY("dirsync",			KEY_KERN_FLAG),
 	FUSE_OPT_KEY("atime",			KEY_KERN_FLAG),
 	FUSE_OPT_KEY("noatime",			KEY_KERN_FLAG),
-	FUSE_OPT_KEY("-h",			KEY_HELP),
-	FUSE_OPT_KEY("--help",			KEY_HELP),
-	FUSE_OPT_KEY("-V",			KEY_VERSION),
-	FUSE_OPT_KEY("--version",		KEY_VERSION),
 	FUSE_OPT_END
 };
 
-static void mount_help(void)
-{
-	fprintf(stderr,
-"    -o allow_other         allow access to other users\n"
-"    -o allow_root          allow access to root\n"
-"    -o auto_unmount        auto unmount on process termination\n"
-"    -o nonempty            allow mounts over non-empty file/dir\n"
-"    -o default_permissions enable permission checking by kernel\n"
-"    -o fsname=NAME         set filesystem name\n"
-"    -o subtype=NAME        set filesystem type\n"
-"    -o large_read          issue large read requests (2.4 only)\n"
-"    -o max_read=N          set maximum size of read requests\n"
-"\n");
-}
-
 static void exec_fusermount(const char *argv[])
 {
 	execv(FUSERMOUNT_DIR "/" FUSERMOUNT_PROG, (char **) argv);
 	execvp(FUSERMOUNT_PROG, (char **) argv);
 }
 
-static void mount_version(void)
+void fuse_mount_version(void)
 {
 	int pid = fork();
 	if (!pid) {
@@ -179,6 +156,11 @@ static const struct mount_flags mount_flags[] = {
 	{NULL,	    0,		    0}
 };
 
+unsigned get_max_read(struct mount_opts *o)
+{
+	return o->max_read;
+}
+
 static void set_mount_flag(const char *s, int *flags)
 {
 	int i;
@@ -200,15 +182,10 @@ static void set_mount_flag(const char *s, int *flags)
 static int fuse_mount_opt_proc(void *data, const char *arg, int key,
 			       struct fuse_args *outargs)
 {
+	(void) outargs;
 	struct mount_opts *mo = data;
 
 	switch (key) {
-	case KEY_ALLOW_ROOT:
-		if (fuse_opt_add_opt(&mo->kernel_opts, "allow_other") == -1 ||
-		    fuse_opt_add_arg(outargs, "-oallow_root") == -1)
-			return -1;
-		return 0;
-
 	case KEY_RO:
 		arg = "ro";
 		/* fall through */
@@ -227,17 +204,9 @@ static int fuse_mount_opt_proc(void *data, const char *arg, int key,
 
 	case KEY_MTAB_OPT:
 		return fuse_opt_add_opt(&mo->mtab_opts, arg);
-
-	case KEY_HELP:
-		mount_help();
-		mo->ishelp = 1;
-		break;
-
-	case KEY_VERSION:
-		mount_version();
-		mo->ishelp = 1;
-		break;
 	}
+
+	/* Pass through unknown options */
 	return 1;
 }
 
@@ -278,7 +247,7 @@ static int receive_fd(int fd)
 	}
 
 	cmsg = CMSG_FIRSTHDR(&msg);
-	if (!cmsg->cmsg_type == SCM_RIGHTS) {
+	if (cmsg->cmsg_type != SCM_RIGHTS) {
 		fprintf(stderr, "got control message of unknown type %d\n",
 			cmsg->cmsg_type);
 		return -1;
@@ -291,9 +260,6 @@ void fuse_kern_unmount(const char *mountpoint, int fd)
 	int res;
 	int pid;
 
-	if (!mountpoint)
-		return;
-
 	if (fd != -1) {
 		struct pollfd pfd;
 
@@ -309,7 +275,8 @@ void fuse_kern_unmount(const char *mountpoint, int fd)
 		close(fd);
 
 		/* If file poll returns POLLERR on the device file descriptor,
-		   then the filesystem is already unmounted */
+		   then the filesystem is already unmounted or the connection
+		   was severed via /sys/fs/fuse/connections/NNN/abort */
 		if (res == 1 && (pfd.revents & POLLERR))
 			return;
 	}
@@ -337,11 +304,6 @@ void fuse_kern_unmount(const char *mountpoint, int fd)
 	waitpid(pid, NULL, 0);
 }
 
-void fuse_unmount_compat22(const char *mountpoint)
-{
-	fuse_kern_unmount(mountpoint, -1);
-}
-
 static int fuse_mount_fusermount(const char *mountpoint, struct mount_opts *mo,
 		const char *opts, int quiet)
 {
@@ -395,7 +357,7 @@ static int fuse_mount_fusermount(const char *mountpoint, struct mount_opts *mo,
 		snprintf(env, sizeof(env), "%i", fds[0]);
 		setenv(FUSE_COMMFD_ENV, env, 1);
 		exec_fusermount(argv);
-		perror("fuse: failed to exec fusermount");
+		perror("fuse: failed to exec fusermount3");
 		_exit(1);
 	}
 
@@ -403,23 +365,21 @@ static int fuse_mount_fusermount(const char *mountpoint, struct mount_opts *mo,
 	rv = receive_fd(fds[1]);
 
 	if (!mo->auto_unmount) {
-		/* with auto_unmount option fusermount will not exit until 
+		/* with auto_unmount option fusermount3 will not exit until
 		   this socket is closed */
 		close(fds[1]);
 		waitpid(pid, NULL, 0); /* bury zombie */
 	}
 
+	if (rv >= 0)
+		fcntl(rv, F_SETFD, FD_CLOEXEC);
+
 	return rv;
 }
 
-int fuse_mount_compat22(const char *mountpoint, const char *opts)
-{
-	struct mount_opts mo;
-	memset(&mo, 0, sizeof(mo));
-	mo.flags = MS_NOSUID | MS_NODEV;
-
-	return fuse_mount_fusermount(mountpoint, &mo, opts, 0);
-}
+#ifndef O_CLOEXEC
+#define O_CLOEXEC 0
+#endif
 
 static int fuse_mount_sys(const char *mnt, struct mount_opts *mo,
 			  const char *mnt_opts)
@@ -444,20 +404,13 @@ static int fuse_mount_sys(const char *mnt, struct mount_opts *mo,
 		return -1;
 	}
 
-	if (!mo->nonempty) {
-		res = fuse_mnt_check_empty("fuse", mnt, stbuf.st_mode,
-					   stbuf.st_size);
-		if (res == -1)
-			return -1;
-	}
-
 	if (mo->auto_unmount) {
-		/* Tell the caller to fallback to fusermount because
+		/* Tell the caller to fallback to fusermount3 because
 		   auto-unmount does not work otherwise. */
 		return -2;
 	}
 
-	fd = open(devname, O_RDWR);
+	fd = open(devname, O_RDWR | O_CLOEXEC);
 	if (fd == -1) {
 		if (errno == ENODEV || errno == ENOENT)
 			fprintf(stderr, "fuse: device not found, try 'modprobe fuse' first\n");
@@ -466,6 +419,8 @@ static int fuse_mount_sys(const char *mnt, struct mount_opts *mo,
 				devname, strerror(errno));
 		return -1;
 	}
+	if (!O_CLOEXEC)
+		fcntl(fd, F_SETFD, FD_CLOEXEC);
 
 	snprintf(tmp, sizeof(tmp),  "fd=%i,rootmode=%o,user_id=%u,group_id=%u",
 		 fd, stbuf.st_mode & S_IFMT, getuid(), getgid());
@@ -508,7 +463,7 @@ static int fuse_mount_sys(const char *mnt, struct mount_opts *mo,
 	if (res == -1) {
 		/*
 		 * Maybe kernel doesn't support unprivileged mounts, in this
-		 * case try falling back to fusermount
+		 * case try falling back to fusermount3
 		 */
 		if (errno == EPERM) {
 			res = -2;
@@ -571,70 +526,79 @@ static int get_mnt_flag_opts(char **mnt_optsp, int flags)
 	return 0;
 }
 
-int fuse_kern_mount(const char *mountpoint, struct fuse_args *args)
+struct mount_opts *parse_mount_opts(struct fuse_args *args)
+{
+	struct mount_opts *mo;
+
+	mo = (struct mount_opts*) malloc(sizeof(struct mount_opts));
+	if (mo == NULL)
+		return NULL;
+
+	memset(mo, 0, sizeof(struct mount_opts));
+	mo->flags = MS_NOSUID | MS_NODEV;
+
+	if (args &&
+	    fuse_opt_parse(args, mo, fuse_mount_opts, fuse_mount_opt_proc) == -1)
+		goto err_out;
+
+	return mo;
+
+err_out:
+	destroy_mount_opts(mo);
+	return NULL;
+}
+
+void destroy_mount_opts(struct mount_opts *mo)
+{
+	free(mo->fsname);
+	free(mo->subtype);
+	free(mo->fusermount_opts);
+	free(mo->subtype_opt);
+	free(mo->kernel_opts);
+	free(mo->mtab_opts);
+	free(mo);
+}
+
+
+int fuse_kern_mount(const char *mountpoint, struct mount_opts *mo)
 {
-	struct mount_opts mo;
 	int res = -1;
 	char *mnt_opts = NULL;
 
-	memset(&mo, 0, sizeof(mo));
-	mo.flags = MS_NOSUID | MS_NODEV;
-
-	if (args &&
-	    fuse_opt_parse(args, &mo, fuse_mount_opts, fuse_mount_opt_proc) == -1)
-		return -1;
-
-	if (mo.allow_other && mo.allow_root) {
-		fprintf(stderr, "fuse: 'allow_other' and 'allow_root' options are mutually exclusive\n");
-		goto out;
-	}
-	res = 0;
-	if (mo.ishelp)
-		goto out;
-
 	res = -1;
-	if (get_mnt_flag_opts(&mnt_opts, mo.flags) == -1)
+	if (get_mnt_flag_opts(&mnt_opts, mo->flags) == -1)
 		goto out;
-	if (mo.kernel_opts && fuse_opt_add_opt(&mnt_opts, mo.kernel_opts) == -1)
+	if (mo->kernel_opts && fuse_opt_add_opt(&mnt_opts, mo->kernel_opts) == -1)
 		goto out;
-	if (mo.mtab_opts &&  fuse_opt_add_opt(&mnt_opts, mo.mtab_opts) == -1)
+	if (mo->mtab_opts &&  fuse_opt_add_opt(&mnt_opts, mo->mtab_opts) == -1)
 		goto out;
 
-	res = fuse_mount_sys(mountpoint, &mo, mnt_opts);
+	res = fuse_mount_sys(mountpoint, mo, mnt_opts);
 	if (res == -2) {
-		if (mo.fusermount_opts &&
-		    fuse_opt_add_opt(&mnt_opts, mo.fusermount_opts) == -1)
+		if (mo->fusermount_opts &&
+		    fuse_opt_add_opt(&mnt_opts, mo->fusermount_opts) == -1)
 			goto out;
 
-		if (mo.subtype) {
+		if (mo->subtype) {
 			char *tmp_opts = NULL;
 
 			res = -1;
 			if (fuse_opt_add_opt(&tmp_opts, mnt_opts) == -1 ||
-			    fuse_opt_add_opt(&tmp_opts, mo.subtype_opt) == -1) {
+			    fuse_opt_add_opt(&tmp_opts, mo->subtype_opt) == -1) {
 				free(tmp_opts);
 				goto out;
 			}
 
-			res = fuse_mount_fusermount(mountpoint, &mo, tmp_opts, 1);
+			res = fuse_mount_fusermount(mountpoint, mo, tmp_opts, 1);
 			free(tmp_opts);
 			if (res == -1)
-				res = fuse_mount_fusermount(mountpoint, &mo,
+				res = fuse_mount_fusermount(mountpoint, mo,
 							    mnt_opts, 0);
 		} else {
-			res = fuse_mount_fusermount(mountpoint, &mo, mnt_opts, 0);
+			res = fuse_mount_fusermount(mountpoint, mo, mnt_opts, 0);
 		}
 	}
 out:
 	free(mnt_opts);
-	free(mo.fsname);
-	free(mo.subtype);
-	free(mo.fusermount_opts);
-	free(mo.subtype_opt);
-	free(mo.kernel_opts);
-	free(mo.mtab_opts);
 	return res;
 }
-
-FUSE_SYMVER(".symver fuse_mount_compat22,fuse_mount@FUSE_2.2");
-FUSE_SYMVER(".symver fuse_unmount_compat22,fuse_unmount@FUSE_2.2");
