@@ -2,10 +2,13 @@
   FUSE: Filesystem in Userspace
   Copyright (C) 2005-2008 Csaba Henk <csaba.henk@creo.hu>
 
+  Architecture specific file system mounting (FreeBSD).
+
   This program can be distributed under the terms of the GNU LGPLv2.
   See the file COPYING.LIB.
 */
 
+#include "config.h"
 #include "fuse_i.h"
 #include "fuse_misc.h"
 #include "fuse_opt.h"
@@ -28,32 +31,23 @@
 #define FUSE_DEV_TRUNK		"/dev/fuse"
 
 enum {
-	KEY_ALLOW_ROOT,
 	KEY_RO,
-	KEY_HELP,
-	KEY_VERSION,
 	KEY_KERN
 };
 
 struct mount_opts {
 	int allow_other;
-	int allow_root;
-	int ishelp;
 	char *kernel_opts;
+	unsigned max_read;
 };
 
-#define FUSE_DUAL_OPT_KEY(templ, key) 				\
+#define FUSE_DUAL_OPT_KEY(templ, key)				\
 	FUSE_OPT_KEY(templ, key), FUSE_OPT_KEY("no" templ, key)
 
 static const struct fuse_opt fuse_mount_opts[] = {
 	{ "allow_other", offsetof(struct mount_opts, allow_other), 1 },
-	{ "allow_root", offsetof(struct mount_opts, allow_root), 1 },
-	FUSE_OPT_KEY("allow_root",		KEY_ALLOW_ROOT),
+	{ "max_read=%u", offsetof(struct mount_opts, max_read), 1 },
 	FUSE_OPT_KEY("-r",			KEY_RO),
-	FUSE_OPT_KEY("-h",			KEY_HELP),
-	FUSE_OPT_KEY("--help",			KEY_HELP),
-	FUSE_OPT_KEY("-V",			KEY_VERSION),
-	FUSE_OPT_KEY("--version",		KEY_VERSION),
 	/* standard FreeBSD mount options */
 	FUSE_DUAL_OPT_KEY("dev",		KEY_KERN),
 	FUSE_DUAL_OPT_KEY("async",		KEY_KERN),
@@ -94,104 +88,37 @@ static const struct fuse_opt fuse_mount_opts[] = {
 	 * handle them
 	 */
 	FUSE_OPT_KEY("fsname=",			KEY_KERN),
-	FUSE_OPT_KEY("nonempty",		KEY_KERN),
-	FUSE_OPT_KEY("large_read",		KEY_KERN),
 	FUSE_OPT_END
 };
 
-static void mount_help(void)
-{
-	fprintf(stderr,
-		"    -o allow_root          allow access to root\n"
-		);
-	system(FUSERMOUNT_PROG " --help");
-	fputc('\n', stderr);
-}
-
-static void mount_version(void)
+void fuse_mount_version(void)
 {
 	system(FUSERMOUNT_PROG " --version");
 }
 
+unsigned get_max_read(struct mount_opts *o)
+{
+	return o->max_read;
+}
+
 static int fuse_mount_opt_proc(void *data, const char *arg, int key,
 			       struct fuse_args *outargs)
 {
 	struct mount_opts *mo = data;
 
 	switch (key) {
-	case KEY_ALLOW_ROOT:
-		if (fuse_opt_add_opt(&mo->kernel_opts, "allow_other") == -1 ||
-		    fuse_opt_add_arg(outargs, "-oallow_root") == -1)
-			return -1;
-		return 0;
-
 	case KEY_RO:
 		arg = "ro";
 		/* fall through */
 
 	case KEY_KERN:
 		return fuse_opt_add_opt(&mo->kernel_opts, arg);
-
-	case KEY_HELP:
-		mount_help();
-		mo->ishelp = 1;
-		break;
-
-	case KEY_VERSION:
-		mount_version();
-		mo->ishelp = 1;
-		break;
 	}
+
+	/* Pass through unknown options */
 	return 1;
 }
 
-void fuse_unmount_compat22(const char *mountpoint)
-{
-	char dev[128];
-	char *ssc, *umount_cmd;
-	FILE *sf;
-	int rv;
-	char seekscript[] =
-		/* error message is annoying in help output */
-		"exec 2>/dev/null; "
-		"/usr/bin/fstat " FUSE_DEV_TRUNK "* | "
-		"/usr/bin/awk 'BEGIN{ getline; if (! ($3 == \"PID\" && $10 == \"NAME\")) exit 1; }; "
-		"              { if ($3 == %d) print $10; }' | "
-		"/usr/bin/sort | "
-		"/usr/bin/uniq | "
-		"/usr/bin/awk '{ i += 1; if (i > 1){ exit 1; }; printf; }; END{ if (i == 0) exit 1; }'";
-
-	(void) mountpoint;
-
-	/*
-	 * If we don't know the fd, we have to resort to the scripted
-	 * solution -- iterating over the fd-s is unpractical, as we
-	 * don't know how many of open files we have. (This could be
-	 * looked up in procfs -- however, that's optional on FBSD; or
-	 * read out from the kmem -- however, that's bound to
-	 * privileges (in fact, that's what happens when we call the
-	 * setgid kmem fstat(1) utility).
-	 */
-	if (asprintf(&ssc, seekscript, getpid()) == -1)
-		return;
-
-	errno = 0;
-	sf = popen(ssc, "r");
-	free(ssc);
-	if (! sf)
-		return;
-
-	fgets(dev, sizeof(dev), sf);
-	rv = pclose(sf);
-	if (rv)
-		return;
-
-	if (asprintf(&umount_cmd, "/sbin/umount %s", dev) == -1)
-		return;
-	system(umount_cmd);
-	free(umount_cmd);
-}
-
 static void do_unmount(char *dev, int fd)
 {
 	char device_path[SPECNAMELEN + 12];
@@ -228,18 +155,21 @@ void fuse_kern_unmount(const char *mountpoint, int fd)
 	(void)mountpoint;
 
 	if (fstat(fd, &sbuf) == -1)
-		return;
+		goto out;
 
 	devname_r(sbuf.st_rdev, S_IFCHR, dev, 128);
 
 	if (strncmp(dev, "fuse", 4))
-		return;
+		goto out;
 
 	strtol(dev + 4, &ep, 10);
 	if (*ep != '\0')
-		return;
+		goto out;
 
 	do_unmount(dev, fd);
+
+out:
+	close(fd);
 }
 
 /* Check if kernel is doing init in background */
@@ -357,32 +287,39 @@ out:
 	return fd;
 }
 
-int fuse_kern_mount(const char *mountpoint, struct fuse_args *args)
+struct mount_opts *parse_mount_opts(struct fuse_args *args)
 {
-	struct mount_opts mo;
-	int res = -1;
+	struct mount_opts *mo;
 
-	memset(&mo, 0, sizeof(mo));
+	mo = (struct mount_opts*) malloc(sizeof(struct mount_opts));
+	if (mo == NULL)
+		return NULL;
+
+	memset(mo, 0, sizeof(struct mount_opts));
+
+	if (args &&
+	    fuse_opt_parse(args, mo, fuse_mount_opts, fuse_mount_opt_proc) == -1)
+		goto err_out;
+
+	return mo;
+
+err_out:
+	destroy_mount_opts(mo);
+	return NULL;
+}
+
+void destroy_mount_opts(struct mount_opts *mo)
+{
+	free(mo->kernel_opts);
+	free(mo);
+}
+
+int fuse_kern_mount(const char *mountpoint, struct mount_opts *mo)
+{
 	/* mount util should not try to spawn the daemon */
 	setenv("MOUNT_FUSEFS_SAFE", "1", 1);
 	/* to notify the mount util it's called from lib */
 	setenv("MOUNT_FUSEFS_CALL_BY_LIB", "1", 1);
 
-	if (args &&
-	    fuse_opt_parse(args, &mo, fuse_mount_opts, fuse_mount_opt_proc) == -1)
-		return -1;
-
-	if (mo.allow_other && mo.allow_root) {
-		fprintf(stderr, "fuse: 'allow_other' and 'allow_root' options are mutually exclusive\n");
-		goto out;
-	}
-	if (mo.ishelp)
-		return 0;
-
-	res = fuse_mount_core(mountpoint, mo.kernel_opts);
-out:
-	free(mo.kernel_opts);
-	return res;
+	return fuse_mount_core(mountpoint, mo->kernel_opts);
 }
-
-FUSE_SYMVER(".symver fuse_unmount_compat22,fuse_unmount@FUSE_2.2");
