@@ -2,10 +2,13 @@
   FUSE: Filesystem in Userspace
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
+  Architecture-independent mounting code.
+
   This program can be distributed under the terms of the GNU LGPLv2.
   See the file COPYING.LIB.
 */
 
+#include "config.h"
 #include "mount_util.h"
 #include <stdio.h>
 #include <unistd.h>
@@ -99,7 +102,13 @@ static int add_mount(const char *progname, const char *fsname,
 		char *env = NULL;
 
 		sigprocmask(SIG_SETMASK, &oldmask, NULL);
-		setuid(geteuid());
+
+		if(setuid(geteuid()) == -1) {
+			fprintf(stderr, "%s: setuid: %s\n", progname, strerror(errno));
+			res = -1;
+			goto out_restore;
+		}
+
 		execle("/bin/mount", "/bin/mount", "--no-canonicalize", "-i",
 		       "-f", "-t", type, "-o", opts, fsname, mnt, NULL, &env);
 		fprintf(stderr, "%s: failed to execute /bin/mount: %s\n",
@@ -152,7 +161,13 @@ static int exec_umount(const char *progname, const char *rel_mnt, int lazy)
 		char *env = NULL;
 
 		sigprocmask(SIG_SETMASK, &oldmask, NULL);
-		setuid(geteuid());
+
+		if(setuid(geteuid()) == -1) {
+			fprintf(stderr, "%s: setuid: %s\n", progname, strerror(errno));
+			res = -1;
+			goto out_restore;
+		}
+
 		if (lazy) {
 			execle("/bin/umount", "/bin/umount", "-i", rel_mnt,
 			       "-l", NULL, &env);
@@ -218,7 +233,13 @@ static int remove_mount(const char *progname, const char *mnt)
 		char *env = NULL;
 
 		sigprocmask(SIG_SETMASK, &oldmask, NULL);
-		setuid(geteuid());
+
+		if(setuid(geteuid()) == -1) {
+			fprintf(stderr, "%s: setuid: %s\n", progname, strerror(errno));
+			res = -1;
+			goto out_restore;
+		}
+
 		execle("/bin/umount", "/bin/umount", "--no-canonicalize", "-i",
 		       "--fake", mnt, NULL, &env);
 		fprintf(stderr, "%s: failed to execute /bin/umount: %s\n",
@@ -312,39 +333,6 @@ char *fuse_mnt_resolve_path(const char *progname, const char *orig)
 	return dst;
 }
 
-int fuse_mnt_check_empty(const char *progname, const char *mnt,
-			 mode_t rootmode, off_t rootsize)
-{
-	int isempty = 1;
-
-	if (S_ISDIR(rootmode)) {
-		struct dirent *ent;
-		DIR *dp = opendir(mnt);
-		if (dp == NULL) {
-			fprintf(stderr,
-				"%s: failed to open mountpoint for reading: %s\n",
-				progname, strerror(errno));
-			return -1;
-		}
-		while ((ent = readdir(dp)) != NULL) {
-			if (strcmp(ent->d_name, ".") != 0 &&
-			    strcmp(ent->d_name, "..") != 0) {
-				isempty = 0;
-				break;
-			}
-		}
-		closedir(dp);
-	} else if (rootsize)
-		isempty = 0;
-
-	if (!isempty) {
-		fprintf(stderr, "%s: mountpoint is not empty\n", progname);
-		fprintf(stderr, "%s: if you are sure this is safe, use the 'nonempty' mount option\n", progname);
-		return -1;
-	}
-	return 0;
-}
-
 int fuse_mnt_check_fuseblk(void)
 {
 	char buf[256];
